#include <BleConnectionStatus.h>
#include <BleKeyboard.h>
#include <KeyboardOutputCallbacks.h>

/* @file MultiKey.ino
  || @version 1.0
  || @author Mark Stanley
  || @contact mstanley@technologist.com
  ||
  || @description
  || | The latest version, 3.0, of the keypad library supports up to 10
  || | active keys all being pressed at the same time. This sketch is an
  || | example of how you can get multiple key presses from a keypad or
  || | keyboard.
  || #
*/

#include <Keypad.h>

BleKeyboard bleKeyboard;

const byte ROWS = 7; //four rows
const byte COLS = 11; //three columns
char keys[ROWS][COLS] = {
  {'y', 'u', 't', 'q', 'e', 'r', 'w', 'C', 'o', 'p', 'i'},
  {'h', 'j', 'g', 'a', 'd', 'f', 's', 'L', 'l', 'D', 'k'},
  {'n', 'm', 'b', 'z', 'c', 'v', 'x', 'F', '.', '/', ','},
  {'P', 'A', 'J', 'H', 'I', ' ', 'N', 'R', 'T', 'V', 'S'},
  {'M', 'X', 'X', '=', 'Z', 'X', 'K', 'G', 'U', 'W', 'E'},
  {'6', '7', '5', '1', '3', '4', '2', 'Q', '9', '0', '8'},
  {'!', '@', '#', '$', '%', '^', '&', '*', 'O', 'Y', 'B'}
};
byte rowPins[ROWS] = {23, 22, 21, 19, 18, 5, 16}; //connect to the row pinouts of the kpd
byte colPins[COLS] = {32, 33, 25, 26, 27, 14, 12, 13, 15, 3, 4}; //connect to the column pinouts of the kpd

Keypad kpd = Keypad( makeKeymap(keys), rowPins, colPins, ROWS, COLS );

unsigned long loopCount;
unsigned long startTime;
String msg;


void setup() {
  Serial.begin(115200);
  Serial.println("Starting BLE work!");
  bleKeyboard.begin();
  startTime = millis();
  msg = "";
}


void loop() {
  if (bleKeyboard.isConnected())
  {
    //Fills kpd.key[ ] array with up - to 10 active keys.
    //Returns true if there are ANY active keys.
    if (kpd.getKeys())
    {
      for (int i = 0; i < LIST_MAX; i++) // Scan the whole key list.
      {
        if (kpd.key[i].stateChanged)   // Only find keys that have changed state.
        {
          switch (kpd.key[i].kchar)
          {
            case 'a':
            case 'b':
            case 'c':
            case 'd':
            case 'e':
            case 'f':
            case 'g':
            case 'h':
            case 'i':
            case 'j':
            case 'k':
            case 'l':
            case 'm':
            case 'n':
            case 'o':
            case 'p':
            case 'q':
            case 'r':
            case 's':
            case 't':
            case 'u':
            case 'v':
            case 'w':
            case 'x':
            case 'y':
            case 'z':
            case '.':
            case '/':
            case ',':
            case ' ':
            case '=':
            case '1':
            case '2':
            case '3':
            case '4':
            case '5':
            case '6':
            case '7':
            case '8':
            case '9':
            case '0':
              if (kpd.key[i].kstate == PRESSED)
              {
                Serial.print(kpd.key[i].kchar);
                bleKeyboard.press(kpd.key[i].kchar);
              }
              if (kpd.key[i].kstate == RELEASED)
              {
                bleKeyboard.release(kpd.key[i].kchar);
              }
              break;
            case '!':
              break;
            case '@':
              break;
            case '#':
              break;
            case '$':
              break;
            case '%':
              break;
            case '^':
              break;
            case '&':
              break;
            case '*':
              break;
            case 'A':
              break;
            case 'B':
              if (kpd.key[i].kstate == PRESSED)
              {
                bleKeyboard.press(KEY_BACKSPACE);
              }
              if (kpd.key[i].kstate == RELEASED)
              {
                bleKeyboard.release(KEY_BACKSPACE);
              }
              break;
            case 'C':
              break;
            case 'D':
              break;
            case 'E':
              if (kpd.key[i].kstate == PRESSED)
              {
                bleKeyboard.press(KEY_RETURN);
              }
              if (kpd.key[i].kstate == RELEASED)
              {
                bleKeyboard.release(KEY_RETURN);
              }
              break;
            case 'F':
              if (kpd.key[i].kstate == PRESSED)
              {
                bleKeyboard.press(KEY_LEFT_SHIFT);
              }
              if (kpd.key[i].kstate == RELEASED)
              {
                bleKeyboard.release(KEY_LEFT_SHIFT);
              }
              break;
            case 'G':
              if (kpd.key[i].kstate == PRESSED)
              {
                bleKeyboard.press(KEY_RIGHT_SHIFT);
              }
              if (kpd.key[i].kstate == RELEASED)
              {
                bleKeyboard.release(KEY_RIGHT_SHIFT);
              }
              break;
            case 'H':
              break;
            case 'I':
              break;
            case 'J':
              break;
            case 'K':
              break;
            case 'L':
              break;
            case 'M':
              break;
            case 'N':
              break;
            case 'O':
              break;
            case 'P':
              break;
            case 'Q':
              if (kpd.key[i].kstate == PRESSED)
              {
                bleKeyboard.press(KEY_ESC);
              }
              if (kpd.key[i].kstate == RELEASED)
              {
                bleKeyboard.release(KEY_ESC);
              }
              break;
            case 'R':
              break;
            case 'S':
              break;
            case 'T':
              if (kpd.key[i].kstate == PRESSED)
              {
                bleKeyboard.press(KEY_UP_ARROW);
              }
              if (kpd.key[i].kstate == RELEASED)
              {
                bleKeyboard.release(KEY_UP_ARROW);
              }
              break;
            case 'U':
              if (kpd.key[i].kstate == PRESSED)
              {
                bleKeyboard.press(KEY_DOWN_ARROW);
              }
              if (kpd.key[i].kstate == RELEASED)
              {
                bleKeyboard.release(KEY_DOWN_ARROW);
              }
              break;
            case 'V':
              if (kpd.key[i].kstate == PRESSED)
              {
                bleKeyboard.press(KEY_LEFT_ARROW);
              }
              if (kpd.key[i].kstate == RELEASED)
              {
                bleKeyboard.release(KEY_LEFT_ARROW);
              }
              break;
            case 'W':
              if (kpd.key[i].kstate == PRESSED)
              {
                bleKeyboard.press(KEY_RIGHT_ARROW);
              }
              if (kpd.key[i].kstate == RELEASED)
              {
                bleKeyboard.release(KEY_RIGHT_ARROW);
              }
              break;
            case 'X':
              break;
            case 'Y':
              break;
            case 'Z':
              break;
          }
        }
      }  // End list of keys
    }  // End active keys
  }  // End connected
}  // End loop
